"""Tests for the sportsball class."""
import unittest

from sportsball.sportsball import SportsBall
from sportsball.data.league import League


class TestSportsball(unittest.TestCase):

    def setUp(self):
        self.sportsball = SportsBall()

    def test_league(self):
        league = self.sportsball.league(League.NFL, None)
        self.assertIsNotNone(league)



"""HKJC HKJC odds model."""

import datetime

from ...bet import Bet
from ...odds_model import OddsModel
from .hkjc_hkjc_bookie_model import create_hkjc_hkjc_bookie_model


def create_hkjc_hkjc_odds_model(odds: float, dt: datetime.datetime) -> OddsModel:
    """Create an odds model with HKJC."""
    return OddsModel(
        odds=odds,
        bookie=create_hkjc_hkjc_bookie_model(),
        dt=dt,
        canonical=False,
        bet=str(Bet.WIN),
    )



"""Tests for the team model class."""
import unittest

from sportsball.data.team_model import TeamModel, VERSION


class TestTeamModel(unittest.TestCase):

    def setUp(self):
        self._team_model = TeamModel(
            identifier="A",
            name="B",
            location=None,
            players=[],
            odds=[],
            points=2.0,
            ladder_rank=10,
            kicks=None,
            news=[],
            social=[],
            field_goals=None,
            coaches=[],
            lbw=None,
            end_dt=None,
            runs=None,
            wickets=None,
            overs=None,
            balls=None,
            byes=None,
            leg_byes=None,
            wides=None,
            no_balls=None,
            penalties=None,
            balls_per_over=None,
            fours=None,
            sixes=None,
            catches=None,
            catches_dropped=None,
            disposal_efficiency=None,
            uncontested_marks=None,
            total_possessions=None,
            tackles_inside_50=None,
            centre_clearances=None,
            stoppage_clearances=None,
            goal_accuracy=None,
            rushed_behinds=None,
            touched_behinds=None,
            left_behinds=None,
            left_posters=None,
            right_behinds=None,
            right_posters=None,
            total_interchange_count=None,
            interchange_count_q1=None,
            interchange_count_q2=None,
            interchange_count_q3=None,
            interchange_count_q4=None,
            game_winning_goals=None,
            headed_goals=None,
            inaccurate_crosses=None,
            inaccurate_long_balls=None,
            inaccurate_passes=None,
            inaccurate_through_balls=None,
            left_footed_shots=None,
            longball_percentage=None,
            offsides=None,
            penalty_kick_goals=None,
            penalty_kick_percentage=None,
            penalty_kick_shots=None,
            penalty_kicks_missed=None,
            possession_percentage=None,
            possession_time=None,
            right_footed_shots=None,
            shoot_out_goals=None,
            shoot_out_misses=None,
            shoot_out_percentage=None,
            shot_assists=None,
            shot_percentage=None,
            shots_headed=None,
            shots_off_target=None,
            shots_on_post=None,
            shots_on_target=None,
            through_ball_percentage=None,
            total_crosses=None,
            total_goals=None,
            total_long_balls=None,
            total_passes=None,
            total_shots=None,
            total_through_balls=None,
            draws=None,
            sub_outs=None,
            suspensions=None,
            time_ended=None,
            time_started=None,
            win_percentage=None,
            wins=None,
            won_corners=None,
            yellow_cards=None,
            clean_sheet=None,
            crosses_caught=None,
            goals_conceded=None,
            partial_clean_sheet=None,
            penalty_kick_conceded=None,
            penalty_kick_save_percentage=None,
            penalty_kicks_faced=None,
            penalty_kicks_saved=None,
            punches=None,
            saves=None,
            shoot_out_kicks_faced=None,
            shoot_out_kicks_saved=None,
            shoot_out_save_percentage=None,
            shots_faced=None,
            smothers=None,
            unclaimed_crosses=None,
            accurate_crosses=None,
            accurate_long_balls=None,
            accurate_passes=None,
            accurate_through_balls=None,
            cross_percentage=None,
            free_kick_goals=None,
            free_kick_percentage=None,
            free_kick_shots=None,
            game_winning_assists=None,
            blocked_shots=None,
            effective_clearances=None,
            effective_tackles=None,
            ineffective_tackles=None,
            interceptions=None,
            tackle_percentage=None,
            appearances=None,
            average_rating_from_correspondent=None,
            average_rating_from_data_feed=None,
            average_rating_from_editor=None,
            average_rating_from_user=None,
            did_not_play=None,
            fouls_committed=None,
            fouls_suffered=None,
            goal_difference=None,
            losses=None,
            lost_corners=None,
            minutes=None,
            own_goals=None,
            pass_percentage=None,
            red_cards=None,
            starts=None,
            sub_ins=None,
            pitch_count=None,
            strikes=None,
            strike_pitch_ratio=None,
            games_played=None,
            team_games_played=None,
            double_plays=None,
            opportunities=None,
            errors=None,
            passed_balls=None,
            outfield_assists=None,
            pickoffs=None,
            putouts=None,
            outs_on_field=None,
            triple_plays=None,
            balls_in_zone=None,
            extra_bases=None,
            outs_made=None,
            catcher_third_innings_played=None,
            catcher_caught_stealing=None,
            catcher_stolen_bases_allowed=None,
            catcher_earned_runs=None,
            is_qualified_catcher=None,
            is_qualified_pitcher=None,
            successful_chances=None,
            total_chances=None,
            full_innings_played=None,
            part_innings_played=None,
            fielding_percentage=None,
            range_factor=None,
            zone_rating=None,
            catcher_caught_stealing_percentage=None,
            catcher_era=None,
            def_warbr=None,
            perfect_games=None,
            wild_pitches=None,
            third_innings=None,
            team_earned_runs=None,
            shutouts=None,
            pickoff_attempts=None,
            run_support=None,
            pitches_as_starter=None,
            quality_starts=None,
            inherited_runners=None,
            inherited_runners_scored=None,
            opponent_total_bases=None,
            is_qualified_saves=None,
            full_innings=None,
            part_innings=None,
            blown_saves=None,
            innings=None,
            era=None,
            whip=None,
            caught_stealing_percentage=None,
            pitches_per_start=None,
            pitches_per_inning=None,
            run_support_average=None,
            opponent_average=None,
            opponent_slug_average=None,
            opponent_on_base_percentage=None,
            opponent_ops=None,
            save_percentage=None,
            strikeouts_per_nine_innings=None,
            strikeout_to_walk_ratio=None,
            tough_losses=None,
            cheap_wins=None,
            save_opportunities_per_win=None,
            runs_created=None,
            batting_average=None,
            pinch_average=None,
            slug_average=None,
            secondary_average=None,
            on_base_percentage=None,
            ops=None,
            ground_to_fly_ratio=None,
            runs_created_per_27_outs=None,
            batter_rating=None,
            at_bats_per_home_run=None,
            stolen_base_percentage=None,
            pitches_per_plate_appearance=None,
            isolated_power=None,
            walk_to_strikeout_ratio=None,
            walks_per_plate_appearance=None,
            secondary_average_minus_batting_average=None,
            runs_produced=None,
            runs_ratio=None,
            patience_ratio=None,
            balls_in_play_average=None,
            mlb_rating=None,
            offensive_wins_above_replacement=None,
            wins_above_replacement=None,
            earned_runs=None,
            batters_hit=None,
            sacrifice_bunts=None,
            save_opportunities=None,
            finishes=None,
            balks=None,
            batters_faced=None,
            holds=None,
            complete_games=None,
            hit_by_pitch=None,
            ground_balls=None,
            strikeouts=None,
            rbis=None,
            sac_hits=None,
            hits=None,
            stolen_bases=None,
            walks=None,
            catcher_interference=None,
            gidps=None,
            sacrifice_flies=None,
            at_bats=None,
            home_runs=None,
            grand_slam_home_runs=None,
            runners_left_on_base=None,
            triples=None,
            game_winning_rbis=None,
            intentional_walks=None,
            doubles=None,
            fly_balls=None,
            caught_stealing=None,
            pitches=None,
            games_started=None,
            pinch_at_bats=None,
            pinch_hits=None,
            player_rating=None,
            is_qualified=None,
            is_qualified_steals=None,
            total_bases=None,
            plate_appearances=None,
            projected_home_runs=None,
            extra_base_hits=None,
            average_game_score=None,
            average_field_goals_attempted=None,
            average_three_point_field_goals_made=None,
            average_three_point_field_goals_attempted=None,
            average_free_throws_made=None,
            average_free_throws_attempted=None,
            average_points=None,
            average_offensive_rebounds=None,
            average_assists=None,
            average_turnovers=None,
            offensive_rebound_percentage=None,
            estimated_possessions=None,
            average_estimated_possessions=None,
            points_per_estimated_possessions=None,
            average_team_turnovers=None,
            average_total_turnovers=None,
            two_point_field_goals_made=None,
            two_point_field_goals_attempted=None,
            average_two_point_field_goals_made=None,
            average_two_point_field_goals_attempted=None,
            two_point_field_goal_percentage=None,
            shooting_efficiency=None,
            scoring_efficiency=None,
            average_48_field_goals_made=None,
            average_48_field_goals_attempted=None,
            average_48_three_point_field_goals_made=None,
            average_48_three_point_field_goals_attempted=None,
            average_48_free_throws_made=None,
            average_48_free_throws_attempted=None,
            average_48_points=None,
            average_48_offensive_rebounds=None,
            average_48_assists=None,
            average_48_turnovers=None,
            average_rebounds=None,
            average_fouls=None,
            average_flagrant_fouls=None,
            average_technical_fouls=None,
            average_ejections=None,
            average_disqualifications=None,
            assist_turnover_ratio=None,
            steal_foul_ratio=None,
            block_foul_ratio=None,
            average_team_rebounds=None,
            total_technical_fouls=None,
            team_assist_turnover_ratio=None,
            steal_turnover_ratio=None,
            average_48_rebounds=None,
            average_48_fouls=None,
            average_48_flagrant_fouls=None,
            average_48_technical_fouls=None,
            average_48_ejections=None,
            average_48_disqualifications=None,
            double_double=None,
            triple_double=None,
            field_goals_made=None,
            free_throws_made=None,
            three_point_percentage=None,
            three_point_field_goals_made=None,
            team_turnovers=None,
            total_turnovers=None,
            points_in_paint=None,
            brick_index=None,
            fast_break_points=None,
            average_field_goals_made=None,
            turnover_points=None,
            average_defensive_rebounds=None,
            average_blocks=None,
            average_steals=None,
            average_48_defensive_rebounds=None,
            average_48_blocks=None,
            average_48_steals=None,
            largest_lead=None,
            disqualifications=None,
            flagrant_fouls=None,
            fouls=None,
            ejections=None,
            technical_fouls=None,
            vorp=None,
            average_minutes=None,
            nba_rating=None,
            fourth_down_attempts=None,
            fourth_down_conversion_percentage=None,
            fourth_down_conversions=None,
            possession_time_seconds=None,
            redzone_attempt_points=None,
            redzone_attempts=None,
            redzone_conversions=None,
            redzone_efficiency_percentage=None,
            redzone_end_downs=None,
            redzone_end_game=None,
            redzone_end_half=None,
            redzone_field_goal_percentage=None,
            redzone_field_goal_points=None,
            redzone_field_goals=None,
            redzone_field_goals_missed=None,
            redzone_fumble=None,
            redzone_interception=None,
            redzone_scoring_percentage=None,
            redzone_total_points=None,
            redzone_touchdown_pass=None,
            redzone_touchdown_percentage=None,
            redzone_touchdown_points=None,
            redzone_touchdown_rush=None,
            redzone_touchdowns=None,
            third_down_attempts=None,
            third_down_conversion_percentage=None,
            third_down_conversions=None,
            timeouts_used=None,
            total_penalties=None,
            total_penalty_yards=None,
            total_plays=None,
            total_drives=None,
            turn_over_differential=None,
            punt_returns_started_inside_the_10=None,
            punt_returns_started_inside_the_20=None,
            punt_return_touchdowns=None,
            punt_return_yards=None,
            special_team_fumble_returns=None,
            special_team_fumble_return_yards=None,
            yards_per_kick_return=None,
            yards_per_punt_return=None,
            yards_per_return=None,
            average_punt_return_yards=None,
            gross_average_punt_yards=None,
            long_punt=None,
            net_average_punt_yards=None,
            punts=None,
            punts_blocked=None,
            punts_blocked_percentage=None,
            punts_inside_10=None,
            punts_inside_10_percentage=None,
            punts_inside_20=None,
            punts_inside_20_percentage=None,
            punts_over_50=None,
            punt_yards=None,
            defensive_points=None,
            kick_extra_points=None,
            kick_extra_points_made=None,
            misc_points=None,
            return_touchdowns=None,
            total_two_point_conversions=None,
            first_downs=None,
            first_downs_passing=None,
            first_downs_penalty=None,
            first_downs_per_game=None,
            first_downs_rushing=None,
            kickoff_return_yards=None,
            kickoffs=None,
            kickoff_yards=None,
            long_field_goal_attempt=None,
            long_field_goal_made=None,
            long_kickoff=None,
            total_kicking_points=None,
            touchback_percentage=None,
            touchbacks=None,
            def_fumble_returns=None,
            def_fumble_return_yards=None,
            fumble_recoveries=None,
            fumble_recovery_yards=None,
            kick_return_fair_catches=None,
            kick_return_fair_catch_percentage=None,
            kick_return_fumbles=None,
            kick_return_fumbles_lost=None,
            kick_returns=None,
            kick_return_touchdowns=None,
            kick_return_yards=None,
            long_kick_return=None,
            long_punt_return=None,
            misc_fumble_returns=None,
            misc_fumble_return_yards=None,
            opp_fumble_recoveries=None,
            opp_fumble_recovery_yards=None,
            opp_special_team_fumble_returns=None,
            opp_special_team_fumble_return_yards=None,
            punt_return_fair_catches=None,
            punt_return_fair_catch_percentage=None,
            punt_return_fumbles=None,
            punt_return_fumbles_lost=None,
            punt_returns=None,
            interception_yards=None,
            average_kickoff_return_yards=None,
            average_kickoff_yards=None,
            extra_point_attempts=None,
            extra_point_percentage=None,
            extra_points_blocked=None,
            extra_points_blocked_percentage=None,
            extra_points_made=None,
            fair_catches=None,
            fair_catch_percentage=None,
            field_goal_attempts=None,
            field_goal_attempts_1_19=None,
            field_goal_attempts_20_29=None,
            field_goal_attempts_30_39=None,
            field_goal_attempts_40_49=None,
            field_goal_attempts_50_59=None,
            field_goal_attempts_60_99=None,
            field_goal_attempts_50=None,
            field_goal_attempt_yards=None,
            field_goals_blocked=None,
            field_goals_blocked_percentage=None,
            field_goals_made_1_19=None,
            field_goals_made_20_29=None,
            field_goals_made_30_39=None,
            field_goals_made_40_49=None,
            field_goals_made_50_59=None,
            field_goals_made_60_99=None,
            field_goals_made_50=None,
            field_goals_made_yards=None,
            field_goals_missed_yards=None,
            kickoff_ob=None,
            kickoff_returns=None,
            kickoff_return_touchdowns=None,
            two_point_rec_conversions=None,
            two_point_reception=None,
            two_point_reception_attempts=None,
            yards_per_reception=None,
            assist_tackles=None,
            average_interception_yards=None,
            average_sack_yards=None,
            average_stuff_yards=None,
            blocked_field_goal_touchdowns=None,
            blocked_punt_touchdowns=None,
            defensive_touchdowns=None,
            hurries=None,
            kicks_blocked=None,
            long_interception=None,
            misc_touchdowns=None,
            passes_batted_down=None,
            passes_defended=None,
            qb_hits=None,
            two_point_returns=None,
            sacks_assisted=None,
            sacks_unassisted=None,
            sack_yards=None,
            safeties=None,
            solo_tackles=None,
            stuff_yards=None,
            tackles_for_loss=None,
            tackles_yards_lost=None,
            yards_allowed=None,
            points_allowed=None,
            one_point_safeties_made=None,
            missed_field_goal_return_td=None,
            blocked_punt_ez_rec_td=None,
            interception_touchdowns=None,
            yards_per_game=None,
            yards_per_pass_attempt=None,
            net_yards_per_pass_attempt=None,
            quarterback_rating=None,
            espn_rb_rating=None,
            long_rushing=None,
            rushing_attempts=None,
            rushing_big_plays=None,
            rushing_first_downs=None,
            rushing_fumbles=None,
            rushing_fumbles_lost=None,
            rushing_touchdowns=None,
            rushing_yards=None,
            rushing_yards_per_game=None,
            stuffs=None,
            stuff_yards_lost=None,
            two_point_rush_conversions=None,
            two_point_rush=None,
            two_point_rush_attempts=None,
            yards_per_rush_attempt=None,
            espn_wr_rating=None,
            long_reception=None,
            receiving_big_plays=None,
            receiving_first_downs=None,
            receiving_fumbles=None,
            receiving_fumbles_lost=None,
            receiving_targets=None,
            receiving_touchdowns=None,
            receiving_yards=None,
            receiving_yards_after_catch=None,
            receiving_yards_at_catch=None,
            receiving_yards_per_game=None,
            receptions=None,
            interception_percentage=None,
            long_passing=None,
            misc_yards=None,
            net_passing_yards=None,
            net_passing_yards_per_game=None,
            net_total_yards=None,
            net_yards_per_game=None,
            passing_attempts=None,
            passing_big_plays=None,
            passing_first_downs=None,
            passing_fumbles=None,
            passing_fumbles_lost=None,
            passing_touchdown_percentage=None,
            passing_touchdowns=None,
            passing_yards=None,
            passing_yards_after_catch=None,
            passing_yards_at_catch=None,
            passing_yards_per_game=None,
            qb_rating=None,
            sacks=None,
            sack_yards_lost=None,
            net_passing_attempts=None,
            total_offensive_plays=None,
            total_points=None,
            total_points_per_game=None,
            total_touchdowns=None,
            total_yards=None,
            total_yards_from_scrimmage=None,
            two_point_pass_convs=None,
            two_point_pass=None,
            two_point_pass_attempts=None,
            yards_from_scrimmage_per_game=None,
            yards_per_completion=None,
            fumbles=None,
            fumbles_lost=None,
            fumbles_forced=None,
            fumbles_recovered_yards=None,
            offensive_two_point_returns=None,
            offensive_fumbles_touchdowns=None,
            defensive_fumbles_touchdowns=None,
            average_gain=None,
            completion_percentage=None,
            completions=None,
            espn_qb_rating=None,
            power_play_time_on_ice=None,
            short_handed_time_on_ice=None,
            even_strength_time_on_ice=None,
            shifts=None,
            shot_differential=None,
            goal_differential=None,
            pim_differential=None,
            rating=None,
            ytd_goals=None,
            shots_in_first_period=None,
            shots_in_second_period=None,
            shots_in_third_period=None,
            shots_ot=None,
            shots_total=None,
            shots_missed=None,
            points_per_game=None,
            power_play_goals=None,
            power_play_assists=None,
            power_play_opportunities=None,
            power_play_percentage=None,
            short_handed_goals=None,
            short_handed_assists=None,
            shootout_attempts=None,
            shootout_shot_percentage=None,
            empty_net_goals_for=None,
            shooting_percentage=None,
            total_face_offs=None,
            faceoffs_won=None,
            faceoffs_lost=None,
            faceoff_percentage=None,
            unassisted_goals=None,
            giveaways=None,
            penalty_minutes=None,
            goals_against=None,
            shots_against=None,
            penalty_kill_percentage=None,
            power_play_goals_against=None,
            short_handed_goals_against=None,
            shootout_saves=None,
            shootout_shots_against=None,
            times_short_handed=None,
            empty_net_goals_against=None,
            takeaways=None,
            even_strength_saves=None,
            power_play_saves=None,
            short_handed_saves=None,
            time_on_ice=None,
            total_giveaways=None,
            total_takeaways=None,
            fantasy_rating=None,
            second_chance_points=None,
            plus_minus=None,
            version=VERSION,
        )

    def test_notnull(self):
        self.assertIsNotNone(self._team_model)



"""The prototype class for a bookie."""

from typing import Literal

from pydantic import BaseModel, ConfigDict, Field

from .field_type import TYPE_KEY, FieldType

BOOKIE_IDENTIFIER_COLUMN: Literal["identifier"] = "identifier"


class BookieModel(BaseModel):
    """The serialisable bookie class."""

    model_config = ConfigDict(
        validate_assignment=False,
        revalidate_instances="never",
        extra="ignore",
        from_attributes=False,
    )

    identifier: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=BOOKIE_IDENTIFIER_COLUMN,
    )
    name: str = Field(..., json_schema_extra={TYPE_KEY: FieldType.TEXT})



"""Tests for the afltables player model class."""
import os
import unittest
import datetime

import requests_mock
import requests_cache
from sportsball.data.afl.afltables.afl_afltables_player_model import create_afl_afltables_player_model


class TestAFLTablesPlayerModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_dt(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "Dennis_Armfield.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/D/Dennis_Armfield.html", content=f.read())
            player_model = create_afl_afltables_player_model(
                player_url="https://afltables.com/afl/stats/players/D/Dennis_Armfield.html",
                jersey="35",
                kicks=10,
                session=self._session,
                name="Dennis Armfield",
                marks=0,
                handballs=0,
                disposals=0,
                goals=0,
                behinds=0,
                hit_outs=0,
                tackles=0,
                rebounds=0,
                insides=0,
                clearances=0,
                clangers=0,
                free_kicks_for=0,
                free_kicks_against=0,
                brownlow_votes=0,
                contested_possessions=0,
                uncontested_possessions=0,
                contested_marks=0,
                marks_inside=0,
                one_percenters=0,
                bounces=0,
                goal_assists=0,
                percentage_played=0.0,
                dt=datetime.datetime(2022, 10, 10),
            )
            self.assertEqual(player_model.identifier, "Dennis_Armfield")

    def test_null_birthdate(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "Jim_Schellnack.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jim_Schellnack.html", content=f.read())
            player_model = create_afl_afltables_player_model(
                player_url="https://afltables.com/afl/stats/players/J/Jim_Schellnack.html",
                jersey="35",
                kicks=10,
                session=self._session,
                name="Dennis Armfield",
                marks=0,
                handballs=0,
                disposals=0,
                goals=0,
                behinds=0,
                hit_outs=0,
                tackles=0,
                rebounds=0,
                insides=0,
                clearances=0,
                clangers=0,
                free_kicks_for=0,
                free_kicks_against=0,
                brownlow_votes=0,
                contested_possessions=0,
                uncontested_possessions=0,
                contested_marks=0,
                marks_inside=0,
                one_percenters=0,
                bounces=0,
                goal_assists=0,
                percentage_played=0.0,
                dt=datetime.datetime(2022, 10, 10),
            )
            self.assertIsNone(player_model.birth_date)

    def test_null_weight(self):
        with requests_mock.Mocker() as m:
            url = "https://afltables.com/afl/stats/players/I/Ivan_Astruc.html"
            with open(os.path.join(self.dir, "Ivan_Astruc.html"), "rb") as f:
                m.get(url, content=f.read())
            player_model = create_afl_afltables_player_model(
                player_url=url,
                jersey="35",
                kicks=10,
                session=self._session,
                name="Dennis Armfield",
                marks=0,
                handballs=0,
                disposals=0,
                goals=0,
                behinds=0,
                hit_outs=0,
                tackles=0,
                rebounds=0,
                insides=0,
                clearances=0,
                clangers=0,
                free_kicks_for=0,
                free_kicks_against=0,
                brownlow_votes=0,
                contested_possessions=0,
                uncontested_possessions=0,
                contested_marks=0,
                marks_inside=0,
                one_percenters=0,
                bounces=0,
                goal_assists=0,
                percentage_played=0.0,
                dt=datetime.datetime(2022, 10, 10),
            )
            self.assertIsNone(player_model.weight)



"""WTA combined league model."""

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...combined.combined_league_model import CombinedLeagueModel
from ...league import League
from ..espn.wta_espn_league_model import WTAESPNLeagueModel
from ..oddsportal.wta_oddsportal_league_model import WTAOddsPortalLeagueModel
from ..tennisabstract.wta_tennisabstract_league_model import \
    WTATennisAbstractLeagueModel

WTA_TEAM_IDENTITY_MAP: dict[str, str] = {}
WTA_VENUE_IDENTITY_MAP: dict[str, str] = {}
WTA_PLAYER_IDENTITY_MAP: dict[str, str] = {}


class WTACombinedLeagueModel(CombinedLeagueModel):
    """WTA combined implementation of the league model."""

    def __init__(self, session: ScrapeSession, league_filter: str | None) -> None:
        super().__init__(
            session,
            League.WTA,
            [
                WTAOddsPortalLeagueModel(session, position=0),
                WTATennisAbstractLeagueModel(session, position=1),
                WTAESPNLeagueModel(session, position=2),
            ],
            league_filter,
        )

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        return WTA_TEAM_IDENTITY_MAP

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        return WTA_VENUE_IDENTITY_MAP

    @classmethod
    def player_identity_map(cls) -> dict[str, str]:
        return WTA_PLAYER_IDENTITY_MAP

    @classmethod
    def name(cls) -> str:
        return "wta-combined-league-model"



"""A function for checking which of the two objects is more interesting."""

# pylint: disable=too-many-return-statements
import datetime
from typing import Any

from .null_check import is_null


def more_interesting(left: Any, right: Any) -> Any:
    """Return the more interesting object."""
    if is_null(left):
        return right
    if is_null(right):
        return left
    if isinstance(left, datetime.datetime) and isinstance(right, datetime.datetime):
        if left.hour == 0 and right.hour != 0:
            return right
        if left.minute == 0 and right.minute != 0:
            return right
        return left
    if isinstance(left, float) and isinstance(right, float):
        if left == 0.0 and right != 0.0:
            return right
        return left
    if isinstance(left, int) and isinstance(right, int):
        if left == 0 and right != 0:
            return right
        return left
    return left



"""NBA NBA.com game model."""

# pylint: disable=too-many-statements,protected-access,too-many-arguments,bare-except,duplicate-code
import datetime
from typing import Any

import extruct  # type: ignore
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...game_model import GameModel
from ...league import League
from ...team_model import VERSION as TEAM_VERSION
from ...venue_model import VERSION
from .nba_nbacom_team_model import create_nba_nbacom_team_model
from .nba_nbacom_venue_model import create_nba_nbacom_venue_model


def create_nba_nbacom_game_model(
    game: dict[str, Any],
    session: ScrapeSession,
    version: str,
) -> GameModel:
    """Create a game model from AFL Tables."""
    game_id = game["gameId"]
    venue_name: str | None = None
    dt = None
    with session.cache_disabled():
        response = session.get(f"https://www.nba.com/game/{game_id}")
        response.raise_for_status()
        data = extruct.extract(response.text, base_url=response.url)
        for jsonld in data["json-ld"]:
            location = jsonld.get("location")
            if location is not None:
                venue_name = location["name"]
            start_date = jsonld.get("startDate")
            if start_date is not None:
                dt = datetime.datetime.fromisoformat(start_date)

    if venue_name is None:
        raise ValueError("venue_name is null")
    if dt is None:
        raise ValueError("dt is null")
    venue_model = create_nba_nbacom_venue_model(
        venue_name=venue_name, session=session, dt=dt, version=VERSION
    )
    return GameModel(
        dt=dt,
        week=None,
        game_number=None,
        venue=venue_model,
        teams=[
            create_nba_nbacom_team_model(
                team=game["homeTeam"], session=session, dt=dt, version=TEAM_VERSION
            ),
            create_nba_nbacom_team_model(
                team=game["awayTeam"], session=session, dt=dt, version=TEAM_VERSION
            ),
        ],
        end_dt=None,
        attendance=None,
        league=League.NBA,
        year=datetime.datetime.today().year,
        season_type=None,
        postponed=None,
        play_off=None,
        distance=None,
        dividends=[],
        pot=None,
        version=version,
        umpires=[],
    )



"""Tests for the ESPN team model class."""
import datetime
import json
import os
import unittest

import requests_mock
from scrapesession.scrapesession import ScrapeSession
from sportsball.data.espn.espn_team_model import create_espn_team_model
from sportsball.data.league import League
from sportsball.data.season_type import SeasonType


class TestESPNTeamModel(unittest.TestCase):

    def setUp(self):
        self._session = ScrapeSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_total_giveaways(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-5.json")) as f:
            statistics_dict = json.load(f)
        roster_dict = {}
        with open(os.path.join(self.dir, "356_roster.json")) as f:
            roster_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "356_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "356_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-7.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/events/401752795/competitions/401752795/competitors/356/roster/4429960/statistics/0?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "356_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons/2025/teams/356/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "559947_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons/2025/coaches/559947?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.NCAAF,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.total_giveaways, 0)

    def test_homeruns(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-8.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "10_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "10_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "10_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/teams/10/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "2_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/teams/2/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "165_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/coaches/165?lang=en&region=us", content=f.read())

            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.MLB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.home_runs, 1)

    def test_even_strength_saves(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-9.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "2_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "2_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "2_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/teams/2/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "2018453_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/coaches/2018453?lang=en&region=us", content=f.read())

            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.NHL,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.even_strength_saves, 0)

    def test_doubles(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-10.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "15_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "15_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "15_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/teams/15/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "157_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/coaches/157?lang=en&region=us", content=f.read())
            

            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.MLB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.doubles, 4)

    def test_second_chance_points(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-11.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "2305_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "2305_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "2305_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/basketball/leagues/mens-college-basketball/seasons/2025/teams/2305/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "71332_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/basketball/leagues/mens-college-basketball/seasons/2025/coaches/71332?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.NCAAB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.second_chance_points, 5.0)

    def test_ytd_goals(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-12.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "23_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "23_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "23_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/teams/23/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "4873586_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/coaches/4873586?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.NHL,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.ytd_goals, 0.0)

    def test_pitcher_notes(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-13.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "19_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "19_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "19_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/teams/19/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "154_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/coaches/154?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.MLB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.name, "Dodgers")

    def test_sac_fly(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-14.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "9_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "9_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "9_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/teams/9/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "170_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/coaches/170?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.MLB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.name, "Twins")

    def test_picked_off(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-16.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "9_score-2.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "9_teams-2.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "9_coaches-2.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/teams/9/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "170_coaches-2.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/baseball/leagues/mlb/seasons/2025/coaches/170?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.MLB,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.name, "Twins")

    def test_plus_minus(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        statistics_dict = {}
        with open(os.path.join(self.dir, "0_statistics-17.json")) as f:
            statistics_dict = json.load(f)
        score_dict = {}
        with open(os.path.join(self.dir, "5_score.json")) as f:
            score_dict = json.load(f)
        team_dict = {}
        with open(os.path.join(self.dir, "5_teams.json")) as f:
            team_dict = json.load(f)
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "5_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/basketball/leagues/wnba/seasons/2025/teams/5/coaches?lang=en&region=us", content=f.read())
            with open(os.path.join(self.dir, "524_coaches.json"), "rb") as f:
                m.get("http://sports.core.api.espn.com/v2/sports/basketball/leagues/wnba/seasons/2025/coaches/524?lang=en&region=us", content=f.read())
            
            team_model = create_espn_team_model(
                session=self._session,
                team=team_dict,
                roster_dict={},
                odds=[],
                score_dict=score_dict,
                dt=dt,
                league=League.WNBA,
                positions_validator={},
                statistics_dict=statistics_dict,
            )
            self.assertEqual(team_model.plus_minus, 0.0)



"""Combined umpire model."""

from typing import Any

from ..umpire_model import VERSION, UmpireModel
from .ffill import ffill
from .most_interesting import more_interesting


def create_combined_umpire_model(
    umpire_models: list[UmpireModel],
    identifier: str,
    umpire_ffill: dict[str, dict[str, Any]],
) -> UmpireModel:
    """Create an umpire model by combining many umpire models."""
    name = None
    sex = None
    birth_date = None
    age = None
    birth_address = None
    high_school = None
    for umpire_model in umpire_models:
        name = more_interesting(name, umpire_model.name)
        sex = more_interesting(sex, umpire_model.sex)
        birth_date = more_interesting(birth_date, umpire_model.birth_date)
        age = more_interesting(age, umpire_model.age)
        birth_address = more_interesting(birth_address, umpire_model.birth_address)
        high_school = more_interesting(high_school, umpire_model.high_school)
    if name is None:
        raise ValueError("name is null")

    umpire_model = UmpireModel.model_construct(
        identifier=identifier,
        name=name,
        sex=sex,
        birth_date=birth_date,
        age=age,
        birth_address=birth_address,
        high_school=high_school,
        version=VERSION,
    )

    ffill(umpire_ffill, identifier, umpire_model)

    return umpire_model



"""Tests for the combined venue model class."""
import os
import unittest

import requests_mock
import requests_cache
from sportsball.data.combined.combined_venue_model import create_combined_venue_model
from sportsball.data.venue_model import VenueModel, VERSION


class TestCombinedVenueModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_is_turf(self):
        is_turf = True
        with requests_mock.Mocker() as m:
            venue_model = VenueModel(
                identifier="a",
                name="The Venue",
                address=None,
                is_grass=None,
                is_indoor=None,
                is_turf=is_turf,
                is_dirt=None,
                version=VERSION,
            )
            venue_model_2 = VenueModel(
                identifier="b",
                name="The Venue",
                address=None,
                is_grass=None,
                is_indoor=None,
                is_turf=None,
                is_dirt=None,
                version=VERSION,
            )
            combined_venue_model = create_combined_venue_model(
                [venue_model, venue_model_2],
                "a",
                self._session,
            )
            self.assertTrue(combined_venue_model.is_turf)



"""AFLW combined league model."""

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...combined.combined_league_model import CombinedLeagueModel
from ...league import League
from ..aflwstats.aflw_aflwstats_league_model import AFLWAFLWStatsLeagueModel
from ..oddsportal.aflw_oddsportal_league_model import AFLWOddsPortalLeagueModel

AFLW_TEAM_IDENTITY_MAP: dict[str, str] = {}
AFLW_VENUE_IDENTITY_MAP: dict[str, str] = {}


class AFLWCombinedLeagueModel(CombinedLeagueModel):
    """AFLW combined implementation of the league model."""

    def __init__(self, session: ScrapeSession, league_filter: str | None) -> None:
        super().__init__(
            session,
            League.AFLW,
            [
                AFLWAFLWStatsLeagueModel(session, position=0),
                AFLWOddsPortalLeagueModel(session, position=1),
            ],
            league_filter,
        )

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        return AFLW_TEAM_IDENTITY_MAP

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        return AFLW_VENUE_IDENTITY_MAP

    @classmethod
    def name(cls) -> str:
        return "aflw-combined-league-model"



"""Tests for the afl AFL game model class."""
import unittest
import os

import requests_cache
from sportsball.data.afl.afl.afl_afl_game_model import _parse


class TestAFLAFLGameModel(unittest.TestCase):

    def setUp(self):
        self.dir = os.path.dirname(__file__)
        self._session = requests_cache.CachedSession(backend="memory")

    def test_parse_game_info(self):
        with open(os.path.join(self.dir, "match.html")) as handle:
            _, _, players = _parse(handle.read(), [], "")
            self.assertEqual(len(players), 2)
            for team in players:
                self.assertEqual(len(team), 26)



"""AFL AFL umpire model."""

from ...umpire_model import VERSION, UmpireModel


def create_afl_afl_umpire_model(
    name: str,
) -> UmpireModel:
    """Create a umpire model from AFL AFL."""
    return UmpireModel(
        identifier=name,
        name=name,
        birth_date=None,
        age=None,
        birth_address=None,
        high_school=None,
        version=VERSION,
    )



"""Tests for the sportsdb venue model class."""
import datetime
import os
import unittest

import requests_mock
from scrapesession.scrapesession import ScrapeSession
from sportsball.data.sportsdb.sportsdb_venue_model import create_sportsdb_venue_model


class TestSportsDBVenueModel(unittest.TestCase):

    def setUp(self):
        self._session = ScrapeSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_identifier(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "venue.json"), "rb") as f:
                m.get("https://www.thesportsdb.com/api/v1/json/3/lookupvenue.php?id=17146", content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=44.5013406&longitude=-88.0647832&start_date=2010-10-09&end_date=2010-10-10&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FChicago&format=flatbuffers")
            venue_model = create_sportsdb_venue_model(
                self._session,
                "19533",
                datetime.datetime(2010, 10, 10, 10, 10, 00),
            )
            self.assertEqual(venue_model.identifier, "17146")



"""The enumeration of the different special field types."""

from enum import StrEnum

TYPE_KEY = "type"
FFILL_KEY = "ffill"


class FieldType(StrEnum):
    """An enumeration over the different field types."""

    LOOKAHEAD = "lookahead"
    CATEGORICAL = "categorical"
    TEXT = "text"
    ODDS = "odds"
    POINTS = "points"
    GOLDEN = "golden"
    DATETIME = "datetime"



"""TennisAbstract league model."""

import os
import urllib.parse
from typing import Iterator

import tqdm
from bs4 import BeautifulSoup
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ..game_model import GameModel
from ..league import League
from ..league_model import SHUTDOWN_FLAG, LeagueModel, needs_shutdown
from .tennisabstract_game_model import create_tennisabstract_game_model


class TennisAbstractLeagueModel(LeagueModel):
    """TennisAbstract implementation of the league model."""

    def __init__(
        self,
        league: League,
        session: ScrapeSession,
        position: int | None = None,
    ) -> None:
        super().__init__(league, session, position=position)

    @classmethod
    def name(cls) -> str:
        """The name of the league model."""
        return "tennisabstract-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        """Tennis position validators."""
        return {}

    @property
    def games(self) -> Iterator[GameModel]:
        """Find all the games."""
        try:
            with tqdm.tqdm(position=self.position) as pbar:
                url = "https://www.tennisabstract.com/charting/"
                response = None
                with self.session.wayback_disabled():
                    with self.session.cache_disabled():
                        self.session.cache.delete(urls=[url])
                        response = self.session.get(url)
                response.raise_for_status()
                soup = BeautifulSoup(response.text, "lxml")
                for a in soup.find_all("a", href=True):
                    if needs_shutdown():
                        return
                    match_name = a.get_text().strip()
                    if self.league == League.ATP:
                        if not match_name.endswith("(ATP)"):
                            continue
                    elif self.league == League.WTA:
                        if not match_name.endswith("(WTA)"):
                            continue
                    match_url = urllib.parse.urljoin(url, a.get("href"))
                    filename = os.path.basename(match_url)
                    datestr = filename.split("-")[0]
                    if len(datestr) == 8 and datestr.isnumeric():
                        pbar.update(1)
                        game_model = create_tennisabstract_game_model(
                            self.session,
                            match_url,
                            self.league,
                        )
                        if game_model is None:
                            continue
                        pbar.set_description(f"TennisAbstract {game_model.dt}")
                        yield game_model
        except Exception as exc:
            SHUTDOWN_FLAG.set()
            raise exc



"""Combined league model."""

# pylint: disable=raise-missing-from,too-many-locals
import datetime
import logging
import multiprocessing
from concurrent.futures import ThreadPoolExecutor, as_completed
from typing import Any, Iterator

import tqdm
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ..game_model import GameModel
from ..league import League
from ..league_model import LeagueModel
from .combined_game_model import create_combined_game_model


def _produce_league_games(league_model: LeagueModel) -> list[dict[str, Any]]:
    return [x.model_dump() for x in league_model.games]


class CombinedLeagueModel(LeagueModel):
    """The class implementing the combined league model."""

    def __init__(
        self,
        session: ScrapeSession,
        league: League,
        league_models: list[LeagueModel],
        league_filter: str | None,
    ) -> None:
        super().__init__(league, session)
        if league_filter is not None:
            league_models = [x for x in league_models if x.name() == league_filter]
        if not league_models:
            raise ValueError("No league models to run")
        self._league_models = league_models

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        """A map to resolve the different teams identities to a consistent identity."""
        raise NotImplementedError(
            "team_identity_map not implemented on CombinedLeagueModel parent class."
        )

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        """A map to resolve the different venue identities to a consistent identity."""
        raise NotImplementedError(
            "venue_identity_map not implemented on CombinedLeagueModel parent class."
        )

    @classmethod
    def player_identity_map(cls) -> dict[str, str]:
        """A map to resolve the different player identities to a consistent identity."""
        return {}

    @property
    def games(self) -> Iterator[GameModel]:
        games: dict[str, list[GameModel]] = {}
        team_identity_map = self.team_identity_map()
        for league_model in self._league_models:
            league_model.clear_session()
        with ThreadPoolExecutor(
            min(multiprocessing.cpu_count(), len(self._league_models))
        ) as p:
            # We want to terminate immediately if any of our runners runs into trouble.

            futures = {
                p.submit(_produce_league_games, model): model
                for model in self._league_models
            }

            results = []
            try:
                for future in as_completed(futures):
                    result = future.result()  # Raises if an exception occurred
                    results.append(result)
            except Exception:
                # Cancel all pending futures
                for f in futures:
                    f.cancel()
                raise  # Optionally re-raise the exception
        game_lists = [[GameModel.model_validate(y) for y in x] for x in results]

        for game_list in game_lists:
            for game_model in game_list:
                old_game_components = [
                    str(game_model.dt.date() - datetime.timedelta(days=1))
                ]
                game_components = [str(game_model.dt.date())]
                for team in game_model.teams:
                    if team.identifier not in team_identity_map:
                        logging.warning(
                            "%s for team %s not found in team identity map.",
                            team.identifier,
                            team.name,
                        )
                    team_identifier = team_identity_map.get(
                        team.identifier, team.identifier
                    )
                    old_game_components.append(team_identifier)
                    game_components.append(team_identifier)
                old_game_components = sorted(game_components)
                game_components = sorted(game_components)
                key = "-".join(old_game_components)
                if key in games:
                    games[key].append(game_model)
                else:
                    key = "-".join(game_components)
                    games[key] = [game_model]
        names: dict[str, str] = {}
        coach_names: dict[str, str] = {}
        player_ffill: dict[str, dict[str, Any]] = {}
        team_ffill: dict[str, dict[str, Any]] = {}
        coach_ffill: dict[str, dict[str, Any]] = {}
        umpire_ffill: dict[str, dict[str, Any]] = {}
        last_game_number = None
        for game_models in tqdm.tqdm(games.values(), desc="Combining Game Models"):
            game_model = create_combined_game_model(  # type: ignore
                game_models=game_models,
                venue_identity_map=self.venue_identity_map(),
                team_identity_map=team_identity_map,
                player_identity_map=self.player_identity_map(),
                session=self.session,
                names=names,
                coach_names=coach_names,
                last_game_number=last_game_number,
                player_ffill=player_ffill,
                team_ffill=team_ffill,
                coach_ffill=coach_ffill,
                umpire_ffill=umpire_ffill,
            )
            last_game_number = game_model.game_number
            yield game_model
